@@ -1,20 +1,22 @@
 #include <boost/python.hpp>
 #include <boost/cstdint.hpp>
+#include <boost/python/suite/indexing/map_indexing_suite.hpp>
+#include <map>
 
 #include <Magick++/Image.h>
+#include <Magick++/STL.h>
+
+#include "_Pixels.h"
 
 using namespace boost::python;
 
 namespace  {
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_adaptiveBlur_overloads_0_2, adaptiveBlur, 0, 2)
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_adaptiveThreshold_overloads_2_3, adaptiveThreshold, 2, 3)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_blur_overloads_0_2, blur, 0, 2)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_blurChannel_overloads_1_3, blurChannel, 1, 3)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_border_overloads_0_1, border, 0, 1)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_charcoal_overloads_0_2, charcoal, 0, 2)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_composite_overloads_3_4, composite, 3, 4)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_composite_overloads_2_3, composite, 2, 3)
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_distort_overloads_3_4, distort, 3, 4)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_edge_overloads_0_1, edge, 0, 1)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_emboss_overloads_0_2, emboss, 0, 2)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_frame_overloads_0_1, frame, 0, 1)
@@ -29,17 +31,49 @@ BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_segment_overloads_0_2, segme
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_shade_overloads_0_3, shade, 0, 3)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_sharpen_overloads_0_2, sharpen, 0, 2)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_sharpenChannel_overloads_1_3, sharpenChannel, 1, 3)
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_sigmoidalContrast_overloads_2_3, sigmoidalContrast, 2, 3)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_solarize_overloads_0_1, solarize, 0, 1)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_spread_overloads_0_1, spread, 0, 1)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_wave_overloads_0_2, wave, 0, 2)
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_signature_overloads_0_1, signature, 0, 1)
+#ifndef PGMAGICK_LIB_GRAPHICSMAGICK_1_3_22
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_adaptiveThreshold_overloads_2_3, adaptiveThreshold, 2, 3)
+#endif
+#ifdef PGMAGICK_LIB_IMAGEMAGICK
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_adaptiveBlur_overloads_0_2, adaptiveBlur, 0, 2)
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_distort_overloads_3_4, distort, 3, 4)
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Magick_Image_sigmoidalContrast_overloads_2_3, sigmoidalContrast, 2, 3)
+#endif
 }
 
+PixelPacketConstArrayProxy get_const_pixels(const Magick::Image* image, int x, int y, unsigned int columns, unsigned int rows) {
+    const Magick::PixelPacket* cache = image->getConstPixels(x, y, columns, rows);
+    return PixelPacketConstArrayProxy(cache, columns*rows, *image);
+}
+
+PixelPacketArrayProxy get_pixels(Magick::Image* image, int x, int y, unsigned int columns, unsigned int rows) {
+    Magick::PixelPacket* cache = image->getPixels(x, y, columns, rows);
+    return PixelPacketArrayProxy(cache, columns*rows, *image);
+}
+
+PixelPacketArrayProxy set_pixels(Magick::Image* image, int x, int y, unsigned int columns, unsigned int rows) {
+    Magick::PixelPacket* cache = image->setPixels(x, y, columns, rows);
+    return PixelPacketArrayProxy(cache, columns*rows, *image);
+}
+
+std::map<Magick::Color,unsigned long> get_color_histogram(const Magick::Image image) {
+    std::map<Magick::Color,unsigned long> histogram;
+    colorHistogram( &histogram, image );
+    return histogram;
+}
 
 void __Image()
 {
     def("InitializeMagick", Magick::InitializeMagick);
+
+    class_<std::map<Magick::Color, unsigned long> >("ColorHistogram")
+         .def(boost::python::map_indexing_suite<std::map<Magick::Color, unsigned long> >())
+    ;
+
     class_< Magick::Image >("Image", init<  >())
         .def(init< const std::string& >())
         .def(init< const Magick::Geometry&, const Magick::Color& >())
@@ -208,6 +242,11 @@ void __Image()
         .def("read", (void (Magick::Image::*)(const Magick::Blob&, const Magick::Geometry&, const std::string&) )&Magick::Image::read)
         .def("reduceNoise", (void (Magick::Image::*)() )&Magick::Image::reduceNoise)
         .def("reduceNoise", (void (Magick::Image::*)(const double) )&Magick::Image::reduceNoise)
+#ifdef PGMAGICK_LIB_GRAPHICSMAGICK_1_3_22   // from 1.3.21
+        .def("resize", (void (Magick::Image::*)(const Magick::Geometry&, const Magick::FilterTypes, const double) )&Magick::Image::resize)
+        .def("resize", (void (Magick::Image::*)(const Magick::Geometry&, const Magick::FilterTypes) )&Magick::Image::resize)
+        .def("resize", (void (Magick::Image::*)(const Magick::Geometry&) )&Magick::Image::resize)
+#endif
         .def("roll", (void (Magick::Image::*)(const Magick::Geometry&) )&Magick::Image::roll)
 #ifdef PGMAGICK_LIB_IMAGEMAGICK
         .def("roll", (void (Magick::Image::*)(const size_t, const size_t) )&Magick::Image::roll)
@@ -215,7 +254,7 @@ void __Image()
         .def("roll", (void (Magick::Image::*)(const unsigned int, const unsigned int) )&Magick::Image::roll)
 #endif
         .def("rotate", &Magick::Image::rotate)
-#ifdef PGMAGICK_LIB_GRAPHICSMAGICK_1_3_19_OVER
+#ifdef PGMAGICK_LIB_GRAPHICSMAGICK_1_3_19
         .def("thumbnail", &Magick::Image::thumbnail)
 #endif
         .def("sample", &Magick::Image::sample)
@@ -285,7 +324,11 @@ void __Image()
         .def("animationIterations", (void (Magick::Image::*)(const unsigned int) )&Magick::Image::animationIterations)
         .def("animationIterations", (unsigned int (Magick::Image::*)() const)&Magick::Image::animationIterations)
 #endif
+#ifdef PGMAGICK_LIB_GRAPHICSMAGICK_1_3_26
+        .def("attribute", (void (Magick::Image::*)(const std::string, const char*) )&Magick::Image::attribute)
+#else
         .def("attribute", (void (Magick::Image::*)(const std::string, const std::string) )&Magick::Image::attribute)
+#endif
         .def("attribute", (std::string (Magick::Image::*)(const std::string) )&Magick::Image::attribute)
         .def("backgroundColor", (void (Magick::Image::*)(const Magick::Color&) )&Magick::Image::backgroundColor)
         .def("backgroundColor", (Magick::Color (Magick::Image::*)() const)&Magick::Image::backgroundColor)
@@ -314,6 +357,7 @@ void __Image()
         .def("clipMask", (Magick::Image (Magick::Image::*)() const)&Magick::Image::clipMask)
         .def("colorFuzz", (void (Magick::Image::*)(const double) )&Magick::Image::colorFuzz)
         .def("colorFuzz", (double (Magick::Image::*)() const)&Magick::Image::colorFuzz)
+        .def("colorHistogram", &get_color_histogram)
 #ifdef PGMAGICK_LIB_IMAGEMAGICK
         .def("colorMap", (void (Magick::Image::*)(const size_t, const Magick::Color&) )&Magick::Image::colorMap)
         .def("colorMap", (Magick::Color (Magick::Image::*)(const size_t) const)&Magick::Image::colorMap)
@@ -536,18 +580,18 @@ void __Image()
         .def("yResolution", &Magick::Image::yResolution)
 #endif
 #if 0
-        .def("getConstPixels",)
         .def("getIndexes",)
         .def("getConstIndexes",)
-        .def("getPixels",)
-        .def("setPixels",)
 #endif
+        .def("getConstPixels", get_const_pixels)
+        .def("getPixels", get_pixels)
+        .def("setPixels", set_pixels)
         .def("syncPixels", &Magick::Image::syncPixels)
         .def("readPixels", &Magick::Image::readPixels)
         .def("writePixels", &Magick::Image::writePixels)
         /* no user-serviceable parts */
         .def("modifyImage", &Magick::Image::modifyImage)
-#if !defined(PGMAGICK_LIB_GRAPHICSMAGICK_1_3_20) && !defined(PGMAGICK_LIB_GRAPHICSMAGICK_1_3_22)
+#if !defined(PGMAGICK_LIB_GRAPHICSMAGICK_1_3_20)
         .def("throwImageException", &Magick::Image::throwImageException)
 #endif
         .def("registerId", &Magick::Image::registerId)
